@@ -80,3 +80,21 @@ br2.data.frame <- function (sim, obs, na.rm=TRUE, ...){
     br2.matrix(sim, obs, na.rm=na.rm, ...)        
      
   } # 'br2.data.frame' END
+  
+  
+################################################################################
+# Author: Mauricio Zambrano-Bigiarini                                          #
+################################################################################
+# Started: 22-Mar-2013                                                         #
+# Updates:                                                                     #
+################################################################################
+br2.zoo <- function(sim, obs, na.rm=TRUE, ...){
+    
+    sim <- zoo::coredata(sim)
+    if (is.zoo(obs)) obs <- zoo::coredata(obs)
+    
+    if (is.matrix(sim) | is.data.frame(sim)) {
+       br2.matrix(sim, obs, na.rm=na.rm, ...)
+    } else NextMethod(sim, obs, na.rm=na.rm, ...)
+     
+  } # 'br2.zoo' end
